struct Player {
	char name[9];
	int namei;
	int score;
	struct sockaddr_in saddr;
	
	int screen[80*24];
	int sentsofar; //bytes of screen
	int sendframe; //frame no TXing now
	int wrkframe;  //frame no in screen

	int key;
};

struct Player* make_player(struct sockaddr_in* saddr)
{
	struct Player* newpl = MY_MALLOC(struct Player, 1);
	newpl->saddr = *saddr;

	newpl->namei = 8;
	newpl->name[8] = 0;
	
	newpl->score = 0;

	newpl->sentsofar = newpl->sendframe = newpl->wrkframe = 0;
	
	newpl->key = ERR;

	return newpl;
}

void kill_player(struct Player* p)
{
	free(p);
}

struct Table {
	char* name;
	struct Deck* d;
	struct Card** cards;
	int n; //num of cards;
	struct Player* lockedby;
	int lockedfor; //time;
	int jiffies;
	
	int screen[80*24];
	int frame; //frame no

	struct Set* set;
	int seti;
};

bool set_exists(struct Table* t)
{
	int i,j,k;
	struct Set* s;
	bool result = false;
	s = make_set();
	for (i = 0; i < 12; ++i) {
		for (j = i + 1; j < 12; ++j) {
			for (k = j + 1; k < 12; ++k) {
				s->cards[0] = t->cards[i];
				s->cards[1] = t->cards[j];
				s->cards[2] = t->cards[k];
				if (good_set(s)) {result = true;break;}
			}
		}
	}
	kill_set(s);
	return result;
}


struct Table* make_table()
{
	int i;
	struct Table* newtable;
	newtable = MY_MALLOC(struct Table, 1);
	newtable->name = my_strdup("default table");
	newtable->d = make_deck();
	newtable->n = 12;
	newtable->cards = MY_MALLOC(struct Card*, newtable->n);
try_again:
	for (i = 0; i < newtable->n; ++i) {
		newtable->cards[i] = get_rand_card(newtable->d);
		newtable->cards[i]->place = i;
	}
	if (!set_exists(newtable)) {
		for (i = 0; i < newtable->n; ++i) putback_card(newtable->cards[i], newtable->d);
		goto try_again;
	}
	newtable->lockedby = NULL;
	newtable->lockedfor = 0;

	newtable->frame = 0;
	newtable->jiffies = 0;

	newtable->set = make_set();
	newtable->seti = 0;
	return newtable;
}

void kill_table(struct Table* t)
{
	 int i;
	 for (i = 0; i < t->n; ++i) putback_card(t->cards[i], t->d);
	 kill_deck(t->d);
	 free(t->name);
	 free(t->set);
	 free(t);
}
/*graphics*/
void draw_box(int X, int Y, int w, int h, int* p)
{
	int x, y;
	for (y = 0; y < h; ++y) {
		for (x = 0; x < w; ++x) {
			p[(Y+y)*80 + X+x] = '.';
		}
	}
}

void draw_text(char* text, int X, int Y, int maxlen, int* p)
{
	while (maxlen--) {
		if (!*text) break;
		p[Y*80 + X++] = *text++;
	}
	
}

void redraw_table(struct Table* t)
{
	
	int X, Y, x, y;
	int i;
fprintf(stderr,"table redraw\n");	
	/* clear */
	for (i = 0; i < 80*24; ++i) t->screen[i] = ' ';
	
	/* draw cards */
	i = 0;
	for (Y = 0; Y < 3; ++Y) {
		for (X = 0; X < 4; ++X) {
			if (t->cards[i]->inset)	draw_box(X*17, Y*6, 17, 7, t->screen);
			for (y = 0; y < 5; ++y) {
				for (x = 0; x < 15; ++x) {
	t->screen[(1 + Y*6  +y)*80 + (1 + X*17  +x)] = t->cards[i]->img[y*15+x];
				}
			}
			++i;		
		}
	}
	/* draw points */
	/* replace sock2pl with player list */
	Y = 0;
	for (i = 0; i < FD_SETSIZE; ++i) {
		char buf[100]; /* ;) */
		struct Player* p = sock2pl[i];
		if (!p) continue;
		snprintf(buf, sizeof(buf), "%s:%i", p->name, p->score);
		draw_text(buf, 68, Y, 11, t->screen);
		++Y;
	}
	t->frame++;
}

void clear_table_set(struct Table* t)
{
	int i;
	for (i = 0; i < t->seti; ++i) {
		t->set->cards[i]->inset = 0;
	}
	t->seti = 0;
	redraw_table(t);
}

void unlock_table(struct Table* t)
{
	t->lockedby = NULL;
	t->lockedfor = 0;
	clear_table_set(t);
fprintf(stderr,"tableunlocked\n");
}


void tick_table(struct Table* t)
{
	if (!t->jiffies) redraw_table(t);
	if (t->lockedby) {
		if (t->lockedfor) t->lockedfor--;
		else {
fprintf(stderr, "forced table unlock\n");
			t->lockedby->score--;
			unlock_table(t);
		}
	}
	t->jiffies++;
/*fprintf(stderr,".");*/
}

void play_table(struct Player* p, struct Table* t)
{
	if (p->namei) { //new player -> nick
		p->name[p->namei-1] = (char)p->key;
		if (!--p->namei) redraw_table(t);
		
		return;
	}
	
	if (!t->lockedby) { /* 0wn the table unless the table already 0wned */
fprintf(stderr,"table locked\n");
		t->lockedby = p;
		t->lockedfor = 5 * HZ;
	}
	if (t->lockedby == p) { /* action allowed for 0wner */
		int i = -1;
		switch(p->key) {
			case 'q': i = 0; break;
			case 'w': i = 1; break;
			case 'e': i = 2; break;
			case 'r': i = 3; break;
			case 'a': i = 4; break;
			case 's': i = 5; break;
			case 'd': i = 6; break;
			case 'f': i = 7; break;
			case 'z': i = 8; break;
			case 'x': i = 9; break;
			case 'c': i = 10; break;
			case 'v': i = 11; break;
		}
		if (i != -1) { /* card selected */
			int j;
			struct Card* newcard = t->cards[i];
fprintf(stderr,"action:%u",i);
			if (!newcard->inset) { /* new card in the set */
fprintf(stderr,"newcardinset");
				newcard->inset = 1;
				t->set->cards[t->seti] = newcard;
				redraw_table(t);
				if (t->seti++ == 2) { /* last card */
fprintf(stderr,"lastcardinset");
					if (good_set(t->set)) { /* set */
fprintf(stderr,"setisgood");
						p->score++;
						do {
							for (j = 0; j < t->set->n; ++j){
								/* swap each card from the set with the new one from the deck */
								int place = t->set->cards[j]->place;
								putback_card(t->set->cards[j], t->d);
								t->set->cards[j] = t->cards[place] = get_rand_card(t->d);
								t->cards[place]->place = place;
							}
fprintf(stderr,"\ntry\n");
						} while (!set_exists(t));
						redraw_table(t);
					} else { /* not really a set */
fprintf(stderr,"notaset");
						p->score--;
					}
					unlock_table(t);
				}

			}
				
		}
		fprintf(stderr,"key %u='%c'\n",p->key,p->key);

	}
}

