/*
    tes - server - see README
    Copyright (C) 2002  Adam Fuksa <fuksa@v-lo.krakow.pl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include<ncurses.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<sys/select.h>
#include<netinet/in.h>
#include<netdb.h>
#include<unistd.h>

#include"card_img.h"

#define HZ 100

#define COL 0
#define SHP 1
#define NUM 2
#define FIL 3

#include "util.c"
#include "card.c"
#include "deck.c"
#include "set.c"

struct Player* sock2pl[FD_SETSIZE];
#include "table.c"

struct Table* tab;
fd_set fds;

void serve(const char* port)
{
	int lsockfd;
	fd_set rfds, wfds, efds;
	struct timeval tv = {0, 0};
	int didsth = 0;
	int ret;
	
	lsockfd = my_listen(port);
	FD_SET(lsockfd, &fds);
	
	for(;;) {
		int i;
		/* receive */
		rfds = fds;
		if (my_select(FD_SETSIZE, &rfds, NULL, NULL, &tv)) {
			didsth = 1;
			for (i = 0; i < FD_SETSIZE; ++i) {
				if (FD_ISSET(i, &rfds)) {
					if (i == lsockfd) { /* new player */
						struct sockaddr_in adr;
						size_t size;
						
						ret = accept(lsockfd, (struct sockaddr*)&adr, &size);
						if (ret == -1) my_die("accept");
					 	sock2pl[ret] = make_player(&adr);
						FD_SET(ret, &fds);
						fprintf(stderr,"new player has arrived..\n");

					} else { /* old player */
						struct Player* p = sock2pl[i];
						ret = recv(i, &p->key, sizeof(p->key), 0);
						if (ret != sizeof(p->key)/*!ret*/) { /* player died */
							fprintf(stderr,"player died\n");
							FD_CLR(i, &fds);
							kill_player(sock2pl[i]);
							sock2pl[i] = NULL;
							continue;
						}
						/*if (ret != sizeof(p->key)) my_die("recv.");*/
						fprintf(stderr,"keypress..\n");
					}
				}
			}
		}
		/* send */
		wfds = fds;
		if (my_select(FD_SETSIZE, NULL, &wfds, NULL, &tv)) {
			for (i = 0; i < FD_SETSIZE; ++i) {
				if (FD_ISSET(i, &wfds)) {
					struct Player* p = sock2pl[i];
					if (p->sentsofar) {
						/* in frame */
						/*XXX can block XXX*/
					goto_hehe:
						ret = send(i, &((char*)p->screen)[p->sentsofar], sizeof(p->screen) - p->sentsofar, 0);
						didsth = 1;
						if (ret == -1) my_die("sent..");
						p->sentsofar += ret;
						if (p->sentsofar == sizeof(p->screen)) p->sentsofar = 0;
						continue;
					} else {
						if (p->sendframe != p->wrkframe) {
							p->sendframe = p->wrkframe;
							goto goto_hehe;
						}
					}
				}
			}
		}
		/* exceptions - what are they? */
		efds = fds;
		if (my_select(FD_SETSIZE, NULL, NULL, &efds, &tv)) {
			didsth = 1;
			fprintf(stderr,"exception\n");
		}
		/* game */
		for (i = 0; i < FD_SETSIZE; ++i) {
			struct Player* p = sock2pl[i];
			if (!p) continue;
			if (p->key!=ERR) {
				didsth = 1;
				play_table(p, tab);
				p->key = ERR;
			}
			if (tab->frame != p->wrkframe) {
fprintf(stderr,"frame copy\n");
				memcpy(p->screen, tab->screen, sizeof(tab->screen));
				p->wrkframe = tab->frame;
			}
		}
		if (!didsth) {
			tick_table(tab);
			usleep (1000000 / HZ);
		}	
		didsth = 0;
	}
}

void init()
{
	int i;

	srand(getpid());
	
	tab = make_table();
	FD_ZERO(&fds);
	for (i = 0; i < FD_SETSIZE; ++i) sock2pl[i] = NULL;
	
}

int main(int argc, char** argv)
{
	if (argc != 2) {
		my_die("argc!=2");
	}
	init();
	serve(argv[1]);
	return 0;
}
